import tkinter as tk
import tkinter.ttk as ttk
import matplotlib
from data import VgSales
import matplotlib.pyplot as plt

matplotlib.use('TkAgg')
from matplotlib.backends.backend_tkagg import FigureCanvasTkAgg


class App(ttk.Frame):

    def __init__(self):
        super().__init__()
        self.root = tk.Tk()
        self.root.title("Video Game Sales")
        self.root.geometry("1200x750")
        self.root.resizable(False, False)
        self.data = VgSales().data
        self.menu_frame = tk.LabelFrame(self.root, text='Menu')
        self.menu_frame.pack(fill="both", expand=True, side=tk.LEFT, padx=5,
                             pady=5)
        self.display_frame = tk.LabelFrame(self.root, text='Display')
        self.display_frame.pack(fill="both", expand=True, side=tk.RIGHT,
                                padx=5, pady=5)
        self.menu()
        self.create_widgets()

    def menu(self):
        self.menu_label1 = tk.Label(self.menu_frame)
        self.menu_label1.grid(sticky='news')

        self.main_combo = ttk.Combobox(self.menu_label1,
                                       values=["Distribution", "Pie Chart",
                                               "Descriptive/Correlation",
                                               "Timeseries"],
                                       state='readonly')
        self.main_combo.bind('<<ComboboxSelected>>', self.update_menu)
        self.main_combo.grid(row=0, column=0, sticky='news', padx=5, pady=5)
        self.main_combo.set('Select activity')

        quit_button = ttk.Button(self.menu_label1, text="Quit",
                                 command=self.root.destroy)
        quit_button.grid(row=0, column=1, sticky='news', padx=5, pady=5)

    def create_widgets(self):
        self.distribution_combo = ttk.Combobox(self.menu_label1,
                                               values=["Publisher", "Genre"],
                                               state='readonly')
        self.distribution_combo.set('Select distribution')
        self.distribution_combo.bind('<<ComboboxSelected>>',
                                     self.update_distribution)
        self.distribution_combo.grid(row=1, column=0, sticky='news', pady=5,
                                     padx=5)
        self.distribution_combo.grid_remove()

        self.des_cor_combo = ttk.Combobox(self.menu_label1,
                                          values=["Descriptive",
                                                  "Correlation"],
                                          state='readonly')
        self.des_cor_combo.set('Select distribution')
        self.des_cor_combo.bind('<<ComboboxSelected>>',
                                self.update_des_cor)
        self.des_cor_combo.grid(row=1, column=0, sticky='news', pady=5,
                                padx=5)
        self.des_cor_combo.grid_remove()

        self.region_combo = ttk.Combobox(self.menu_label1,
                                         values=[region for region in
                                                 VgSales().get_region()],
                                         state='readonly')
        self.region_combo.set('Select Region')
        self.region_combo.bind('<<ComboboxSelected>>', self.update_region)
        self.region_combo.grid(row=1, column=0, sticky='news', pady=5, padx=5)
        self.region_combo.grid_remove()

        self.sub_label1 = tk.Label(self.menu_label1,
                                   text="Make sure to select all attributes!",
                                   fg="white")
        self.sub_label1.grid(row=4, sticky='w', padx=5, pady=5)
        self.show_button = tk.Button(self.menu_label1, text='Show')
        self.show_button.grid(row=4, column=1, sticky='w', pady=5, padx=5)

    def update_menu(self, event):
        choice = self.main_combo.get()
        if choice == "Distribution":
            self.combo_reset()
            self.distribution_combo.grid()
            self.des_cor_combo.grid_remove()
            self.region_combo.grid_remove()
            self.show_button.configure(command=self.Distribution)
        elif choice == "Pie Chart":
            self.combo_reset()
            self.distribution_combo.grid()
            self.des_cor_combo.grid_remove()
            self.region_combo.grid_remove()
            self.show_button.configure(command=self.Piechart)
        elif choice == "Descriptive/Correlation":
            self.combo_reset()
            self.distribution_combo.grid_remove()
            self.des_cor_combo.grid()
            self.region_combo.grid_remove()
            self.show_button.configure(command=self.Descriptive)
        elif choice == "Timeseries":
            self.combo_reset()
            self.region_combo.grid()
            self.des_cor_combo.grid_remove()
            self.distribution_combo.grid_remove()
            self.show_button.configure(command=self.Timeseries)
        else:
            self.combo_reset()
            self.des_cor_combo.grid_remove()
            self.distribution_combo.grid_remove()
            self.region_combo.grid_remove()

    def combo_reset(self):
        self.distribution_combo.set('Select distribution')
        self.region_combo.set('Select Region')
        self.show_button.configure(command=None)
        self.distribution = None
        self.region = None
        self.des_cor = None

    def update_distribution(self, event):
        self.distribution = self.distribution_combo.get()

    def update_region(self, event):
        self.region = self.region_combo.get()

    def update_des_cor(self, event):
        self.des_cor = self.des_cor_combo.get()

    def Distribution(self):
        if self.distribution == None:
            print('Please select all attributes!')
        elif self.distribution == "Publisher":
            self.reset_frame()
            data = self.data["Publisher"]
            frequency = data.value_counts().sort_index()
            distribution = VgSales().get_publisher()
            fig = plt.figure()
            ax = fig.add_axes([0.1, 0.3, 0.8, 0.6])
            ax.bar(distribution, frequency, color='blue')
            ax.set_xlabel("Publisher")
            ax.set_ylabel('Frequency')
            ax.set_title("Publisher Distribution")
            plt.xticks(rotation=90, fontsize=7)
            canvas = FigureCanvasTkAgg(fig, self.display_frame)
            canvas.draw()
            canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)
        elif self.distribution == "Genre":
            self.reset_frame()
            data = self.data["Genre"]
            frequency = data.value_counts().sort_index()
            distribution = VgSales().get_genre()
            fig = plt.figure()
            ax = fig.add_axes([0.1, 0.16, 0.8, 0.74])
            ax.bar(distribution, frequency, color='blue')
            ax.set_xlabel("Genre")
            ax.set_ylabel('Frequency')
            ax.set_title("Genre Distribution")
            plt.xticks(rotation=90)
            canvas = FigureCanvasTkAgg(fig, self.display_frame)
            canvas.draw()
            canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def Piechart(self):
        if self.distribution == None:
            print('Please select all attributes!')
        elif self.distribution == "Publisher":
            self.reset_frame()
            data = self.data["Publisher"]
            frequency = data.value_counts().sort_index()
            fig = plt.figure()
            ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
            ax.pie(frequency, labels=VgSales().get_publisher(),
                   autopct='%1.1f%%', startangle=90)
            ax.axis('equal')
            ax.set_title('Portion of Publisher')
            canvas = FigureCanvasTkAgg(fig, self.display_frame)
            canvas.draw()
            canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)
        elif self.distribution == "Genre":
            self.reset_frame()
            data = self.data["Genre"]
            frequency = data.value_counts().sort_index()
            fig = plt.figure()
            ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
            ax.pie(frequency, labels=VgSales().get_genre(),
                   autopct='%1.1f%%', startangle=90)
            ax.axis('equal')
            ax.set_title('Portion of Genre')
            canvas = FigureCanvasTkAgg(fig, self.display_frame)
            canvas.draw()
            canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def Descriptive(self):
        if self.des_cor == None:
            print('Please select all attributes!')
        elif self.des_cor == "Descriptive":
            self.reset_frame()
            data = self.data['Global_Sales']
            title = tk.Label(self.display_frame,
                             text='Descriptive statistic of Global Sales',
                             font=('Ariel', 30))
            title.pack()
            mean = tk.Label(self.display_frame,
                            text=f'Mean: {data.mean():.2f}',
                            font=('Ariel', 20))
            mean.pack()
            median = tk.Label(self.display_frame,
                              text=f'Median: {data.median():.2f}',
                              font=('Ariel', 20))
            median.pack()
            std = tk.Label(self.display_frame,
                           text=f'Standard deviation: {data.std():.2f}',
                           font=('Ariel', 20))
            std.pack()
            q1 = tk.Label(self.display_frame,
                          text=f'1st-quartile: {data.quantile(0.25):.2f}',
                          font=('Ariel', 20))
            q1.pack()
            q3 = tk.Label(self.display_frame,
                          text=f'3rd-quartile: {data.quantile(0.75):.2f}',
                          font=('Ariel', 20))
            q3.pack()
        elif self.des_cor == "Correlation":
            self.reset_frame()
            global_sales = self.data['Global_Sales']
            na_sales = self.data['NA_Sales']
            fig = plt.figure()
            plt.scatter(global_sales, na_sales, c='blue')
            plt.xlabel('Global Sales')
            plt.ylabel('NA Sales')
            plt.title('Relationship between Global Sales and NA Sales')
            canvas = FigureCanvasTkAgg(fig, self.display_frame)
            canvas.draw()
            canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def Timeseries(self):
        if self.region == None:
            print("Please select all attributes!")
        else:
            self.reset_frame()
            year = VgSales().get_year()
            sales = self.data.groupby(by=['Year'])[self.region].sum()
            fig = plt.figure()
            ax = fig.add_axes([0.1, 0.1, 0.8, 0.8])
            line1, = ax.plot(year, sales)
            line1.set_color('b')
            line1.set_marker('o')
            line1.set_linewidth(1)
            ax.set_xlabel('Year')
            ax.set_ylabel(self.region)
            ax.set_title(f'Timeseries of {self.region}')
            canvas = FigureCanvasTkAgg(fig, self.display_frame)
            canvas.draw()
            canvas.get_tk_widget().pack(fill=tk.BOTH, expand=True)

    def reset_frame(self):
        for graph in self.display_frame.winfo_children():
            graph.destroy()

    def run_app(self):
        self.mainloop()

